//+------------------------------------------------------------------+
//|                                                     JLPiGrid.mq4 |
//|                                           Copyright  2007, jlpi |
//|                      http://www.fx-trading-education.com/mt4.html|
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, jlpi"
#property link      "http://www.fx-trading-education.com/mt4.html"

#include <stdlib.mqh>

#define MAGICMABO  19665

extern double  TrendInterval = 70;
extern int     Spread = 4;
extern int     MAPeriod = 75;
extern int     ADXThreshold = 20;
extern double  Percent = 5;
extern double  MultiplierTrendInterval = 1.4;
extern double  Risk = 0.1;
extern bool    MicroLots = true;
extern double  MinLots = 0.01;
extern int     StopLoss = 0;

double Multi = 0;
double Interval = 0;

double MaxLots     = 5.0;           
double Slippage     = 3;

double Init = 0;
double NextUp;
double NextDown;
double NextDownUp;
double NextUpDown;
double RefBalance;

bool     BuyOnly = false;
bool     SellOnly = false;

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init() 
  {
     Init = 0;
     RefBalance = AccountBalance();
     GlobalVariableSet("RefBalance", AccountBalance());
     GlobalVariableSet("Init", 0);

     if (Symbol() == "USDJPY" || Symbol() == "EURJPY" || Symbol() == "GBPJPY" || 
         Symbol() == "CHFJPY" || Symbol() == "USDJPYm" || Symbol() == "EURJPYm" || 
         Symbol() == "GBPJPYm" || Symbol() == "CHFJPYm")
         Multi = 100;
     else
         Multi = 10000;
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit() 
  {
   return(0);
  }


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
   {
   int      cnt, ticket, err, cmd, i;
   double   ts, tp, Price;
   string   setup;
   double   Lots;
   double   ma, maprev;
   double   adx;
   double   sl = 0;
   string   text = ""; 
     
   RefBalance = GlobalVariableGet("RefBalance");
   Init = GlobalVariableGet("Init");

   setup="JLPi_Grid";

   if (MicroLots)   
      Lots = MathRound(AccountFreeMargin()*Risk/100)/100;
   else
      Lots = MathRound(AccountFreeMargin()*Risk/1000)/10;
   if (Lots > MaxLots)
     Lots = MaxLots;  
   text = text + "\nLots = " + Lots;   

   if (Lots < MinLots)
   {
      Comment("Lots = ", Lots, " is less then MinLots");
      return(0);
   }
       
   if (AccountFreeMargin() < (1000*Lots))
   {
      Comment("Lots = ", Lots, "x 1000 = ", Lots*1000," is less then Free Margin");
      return(0);    
   }
        
   adx=iADX(Symbol(),PERIOD_D1,14,PRICE_TYPICAL,MODE_MAIN,1);
   ma = iMA(Symbol(),PERIOD_D1,MAPeriod,0,MODE_SMA,PRICE_TYPICAL,1);
   maprev = iMA(Symbol(),PERIOD_D1,MAPeriod,0,MODE_SMA,PRICE_TYPICAL,2);
   text = text + "\nADX = " + adx + " - MA1 = " + ma + " - MA2 = " + maprev;
      
   if (AccountEquity() >=  RefBalance*(1.0 + Percent/100.0))
   {
      Init = 0;
      BuyOnly = false;
      SellOnly = false;
      for(cnt=OrdersTotal();cnt>=0;cnt--)
      {
         if (OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES)) {     
            if(OrderType()==OP_BUY && OrderMagicNumber()==MAGICMABO) {
         		 OrderClose(OrderTicket(), OrderLots(), Bid, 3, Red);
            } else 
            if(OrderType()==OP_SELL && OrderMagicNumber()==MAGICMABO) {
         		    OrderClose(OrderTicket(), OrderLots(), Ask, 3, Red);
		      }
         } 
      }
   }
     
   if (Init == 0) // init of the grid
   {
      Init = 1;
      
      // init next values

      Interval = TrendInterval;
      if (ma>maprev && adx > ADXThreshold)
            BuyOnly = true;
      else if (ma<maprev && adx > ADXThreshold)
            SellOnly = true;
      else
      {
         BuyOnly = true;
         SellOnly = true;
         Interval = MathRound(TrendInterval*MultiplierTrendInterval);
      }
            
      NextDown = MathRound((Bid-(Interval/2*Point))*Multi/Interval)/(Multi/Interval);
      NextUp = NextDown + Interval*Point;
      NextDownUp = NextUp;
      NextUpDown = NextDown;
      RefBalance = AccountBalance();

      GlobalVariableSet("RefBalance", AccountBalance());
      GlobalVariableSet("Init", 1);           
   }

   text = text + "\nOperation = ";
   if (BuyOnly) text = text + "Buy ";
   if (SellOnly) text = text + "Sell";
         
   text = text + "\nInterval = " + Interval;
   text = text + "\nNext Down = " + NextDown + " - Next Up = " + NextUp ;
   text = text + "\nNext DownUp = " + NextDownUp + " - Next UpDown = " + NextUpDown ;
   text = text + "\nReference Balance  = " + RefBalance;

   // -------------------------------------------------
   //          P U T T I N G   O R D E R S        	    
   // -------------------------------------------------
   
   if (Bid >= NextUp) // Next stage up 
   {
      // add one more buy order
      if (BuyOnly) {

          if (StopLoss>0)     sl = Ask - StopLoss * Point;
		    ticket = OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,sl,
		             Ask+(Interval-Spread)*Point,setup,MAGICMABO,0,Green);
		    if (ticket == -1)
		    {
		        err = GetLastError();
		        Print("Error setting buy order [" + setup + "]: (" + err + ") " + 
		               ErrorDescription(err));
		    }
		}
    
      if (SellOnly) {
          if (StopLoss>0)
            sl = Bid + StopLoss * Point;
		    ticket =  OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,sl,
		              Bid-(Interval-Spread)*Point,setup,MAGICMABO,0,Green);
		    if (ticket == -1)
		    {
		        err = GetLastError();
		        Print("Error setting sell order [" + setup + "]: (" + err + ") " + 
		               ErrorDescription(err));
		    }
		}
    
    //replacing thresholds
     NextUp = NextUp + Interval*Point;
     NextUpDown = NextUp - 2*Interval*Point;
     NextDownUp = NextUp;
  }


   if (Ask <= NextDown) // Next stage Down 
   {
      // add one more buy order
      if (BuyOnly) {
          if (StopLoss>0)
            sl = Ask - StopLoss * Point;
		    ticket = OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,sl,
		             Ask+(Interval-Spread)*Point,setup,MAGICMABO,0,Green);
		    if (ticket == -1)
		    {
		        err = GetLastError();
		        Print("Error setting buy order [" + setup + "]: (" + err + ") " + 
		               ErrorDescription(err));
		    }
		}

      if (SellOnly) {
          if (StopLoss>0)
            sl = Bid + StopLoss * Point;
		    ticket = OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,sl,
		             Bid-(Interval-Spread)*Point,setup,MAGICMABO,0,Green);
		    if (ticket == -1)
		    {
		        err = GetLastError();
		        Print("Error setting sell order [" + setup + "]: (" + err + ") " + 
		               ErrorDescription(err));
		    }
		}

    //replacing thresholds    
     NextDown = NextDown - Interval*Point;
     NextDownUp = NextDown + 2*Interval*Point;
     NextUpDown = NextDown;
  }


   if (Bid < NextUp && Bid >= NextDownUp) // Coming back up
   {
      // buy order exists already
      if (SellOnly) {
          if (StopLoss>0)
            sl = Bid + StopLoss * Point;
		    ticket = OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,sl,
		             Bid-(Interval-Spread)*Point,setup,MAGICMABO,0,Green);
		    if (ticket == -1)
		    {
		        err = GetLastError();
		        Print("Error setting sell order [" + setup + "]: (" + err + ") " + 
		               ErrorDescription(err));
		    }
		}
    
    //replacing thresholds    
     NextDownUp = NextDownUp + Interval*Point;
     NextUpDown = NextDownUp - 2*Interval*Point;
   }


   if (Ask > NextDown && Ask <= NextUpDown) // Coming back down
   {
      // sell order exists already
      if (BuyOnly) {
          if (StopLoss>0)
            sl = Ask - StopLoss * Point;
		    ticket = OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,sl,
		             Ask+(Interval-Spread)*Point,setup,MAGICMABO,0,Green);
		    if (ticket == -1)
		    {
		        err = GetLastError();
		        Print("Error setting buy order [" + setup + "]: (" + err + ") " + 
		               ErrorDescription(err));
		    }
		}

    //replacing thresholds    
     NextUpDown = NextUpDown - Interval*Point;
     NextDownUp = NextUpDown + 2*Interval*Point;
   }


   Comment(text);  
   return(0);

}


